@@ -17,10 +17,9 @@ import os
 import warnings
 from abc import ABCMeta, abstractmethod
 from collections import defaultdict
-from copy import deepcopy
 from pathlib import Path
 from typing import (Any, Callable, DefaultDict, Dict, List, Optional,
-                    OrderedDict, Tuple, Union)
+                    OrderedDict, Sequence, Tuple, Union, cast)
 
 from PIL import Image
 from pydicom.dataset import FileMetaDataset
@@ -38,7 +37,7 @@ from wsidicom.instance import (ImageData, WsiDataset, WsiDicomFile,
 from wsidicom.optical import OpticalManager
 from wsidicom.stringprinting import (dict_pretty_str, list_pretty_str,
                                      str_indent)
-from wsidicom.uid import ANN_SOP_CLASS_UID, WSI_SOP_CLASS_UID, BaseUids
+from wsidicom.uid import ANN_SOP_CLASS_UID, WSI_SOP_CLASS_UID, SlideUids
 
 
 class WsiDicomGroup:
@@ -46,14 +45,14 @@ class WsiDicomGroup:
     different z coordinate and/or optical path."""
     def __init__(
         self,
-        instances: List[WsiInstance]
+        instances: Sequence[WsiInstance]
     ):
         """Create a group of WsiInstances. Instances should match in the common
         uids, wsi type, and tile size.
 
         Parameters
         ----------
-        instances: List[WsiInstance]
+        instances: Sequence[WsiInstance]
             Instances to build the group.
         """
         self._instances = {  # key is identifier (Uid)
@@ -96,7 +95,7 @@ class WsiDicomGroup:
         return self.instances[index]
 
     @property
-    def uids(self) -> BaseUids:
+    def uids(self) -> SlideUids:
         """Return uids"""
         return self._uids
 
@@ -167,13 +166,13 @@ class WsiDicomGroup:
     @classmethod
     def open(
         cls,
-        instances: List[WsiInstance],
+        instances: Sequence[WsiInstance],
     ) -> List['WsiDicomGroup']:
         """Return list of groups created from wsi instances.
 
         Parameters
         ----------
-        files: List[WsiInstance]
+        files: Sequence[WsiInstance]
             Instances to create groups from.
 
         Returns
@@ -192,11 +191,21 @@ class WsiDicomGroup:
         return groups
 
     def matches(self, other_group: 'WsiDicomGroup') -> bool:
-        """Check if group is valid (Uids and tile size match).
-        The common Uids should match for all series.
+        """Check if group matches other group. If strict common Uids should
+        match. Wsi type should always match.
+
+        Parameters
+        ----------
+        other_group: WsiDicomGroup
+            Other group to match against.
+
+        Returns
+        ----------
+        bool
+            True if other group matches.
         """
         return (
-            other_group.uids == self.uids and
+            self.uids.matches(other_group.uids) and
             other_group.wsi_type == self.wsi_type
         )
 
@@ -438,28 +447,28 @@ class WsiDicomGroup:
             instance.close()
 
     def _validate_group(self):
-        """Check that no file or instance in group is duplicate, instances in
-        group matches and that the optical manager matches by base uid.
-        Raises WsiDicomMatchError otherwise.
+        """Check that no file or instance in group is duplicate, and if strict
+        instances in group matches. Raises WsiDicomMatchError otherwise.
         """
-        WsiDataset.check_duplicate_dataset(self.datasets, self)
         instances = list(self.instances.values())
         base_instance = instances[0]
         for instance in instances[1:]:
             if not base_instance.matches(instance):
                 raise WsiDicomMatchError(str(instance), str(self))
+
+        WsiDataset.check_duplicate_dataset(self.datasets, self)
         WsiInstance.check_duplicate_instance(instances, self)
 
     @classmethod
     def _group_instances(
         cls,
-        instances: List[WsiInstance]
+        instances: Sequence[WsiInstance]
     ) -> OrderedDict[Size, List[WsiInstance]]:
         """Return instances grouped and sorted by image size.
 
         Parameters
         ----------
-        instances: List[WsiInstance]
+        instances: Sequence[WsiInstance]
             Instances to group by image size.
 
         Returns
@@ -491,9 +500,11 @@ class WsiDicomGroup:
         List[List[WsiInstance]]
             Instances grouped by common properties.
         """
-        groups: DefaultDict[Any, List[WsiInstance]] = (
-            defaultdict(list)
-        )
+        groups: DefaultDict[
+            Tuple[str, UID, bool, Optional[int], Optional[float], str],
+            List[WsiInstance]
+        ] = defaultdict(list)
+
         for instance in self.instances.values():
             groups[
                 instance.image_data.photometric_interpretation,
@@ -501,8 +512,7 @@ class WsiDicomGroup:
                 instance.ext_depth_of_field,
                 instance.ext_depth_of_field_planes,
                 instance.ext_depth_of_field_plane_distance,
-                instance.focus_method,
-                instance.slice_spacing
+                instance.focus_method
             ].append(
                 instance
             )
@@ -510,20 +520,20 @@ class WsiDicomGroup:
 
     @staticmethod
     def _list_image_data(
-        instances: List[WsiInstance]
-    ) -> List[Tuple[Tuple[str, float], ImageData]]:
+        instances: Sequence[WsiInstance]
+    ) -> OrderedDict[Tuple[str, float], ImageData]:
         """Sort ImageData in instances by optical path and focal
         plane.
 
         Parameters
         ----------
-        instances: List[WsiInstance]
+        instances: Sequence[WsiInstance]
             List of instances with optical paths and focal planes to list and
             sort.
 
         Returns
         ----------
-        Tuple[Tuple[str, float], ImageData]
+        OrderedDict[Tuple[str, float], ImageData]:
             ImageData sorted by optical path and focal plane.
         """
         output: Dict[Tuple[str, float], ImageData] = {}
@@ -532,37 +542,15 @@ class WsiDicomGroup:
                 for z in instance.focal_planes:
                     if (optical_path, z) not in output:
                         output[optical_path, z] = instance.image_data
-        return list(OrderedDict(output).items())
-
-    @staticmethod
-    def _get_number_of_frames(
-        instances: List[WsiInstance]
-    ) -> int:
-        """
-
-        Parameters
-        ----------
-        instances: List[WsiInstance]
-
-
-        Returns
-        ----------
-        int
-        """
-        frames = 0
-        for instance in instances:
-            tiles = instance.image_data.tiled_size.area
-            optical_paths = len(instance.optical_paths)
-            focal_planes = len(instance.focal_planes)
-            frames += tiles*optical_paths*focal_planes
-        return frames
+        return OrderedDict(output)
 
     def save(
         self,
         output_path: str,
         uid_generator: Callable[..., UID],
         workers: int,
-        chunk_size: int
+        chunk_size: int,
+        offset_table: Optional[str]
     ) -> List[Path]:
         """Save a WsiDicomGroup to files in output_path. Instances are grouped
         by properties that cant differ in the same file:
@@ -570,7 +558,6 @@ class WsiDicomGroup:
             - transfer syntax
             - extended depth of field (and planes and distance)
             - focus method
-            - spacing between slices
         Other properties are assumed to be equal or to be updated.
 
         Parameters
@@ -584,6 +571,9 @@ class WsiDicomGroup:
         chunk_size: int
             Chunk size (number of tiles) to process at a time. Actual chunk
             size also depends on minimun_chunk_size from image_data.
+        offset_table: Optional[str]
+            Offset table to use, 'bot' basic offset table, 'eot' extended
+            offset table, None - no offset table.
 
         Returns
         ----------
@@ -593,58 +583,23 @@ class WsiDicomGroup:
         filepaths: List[Path] = []
         for instances in self._group_instances_to_file():
             uid = uid_generator()
-            if output_path == os.devnull:
-                filepath = Path(output_path)
-            else:
-                filepath = Path(os.path.join(output_path, uid + '.dcm'))
-
+            filepath = Path(os.path.join(output_path, uid + '.dcm'))
             transfer_syntax = instances[0].image_data.transfer_syntax
-            dataset = deepcopy(instances[0].dataset)
-            frames = self._get_number_of_frames(instances)
-            dataset.NumberOfFrames = frames
-            self._write_file(
-                filepath,
-                uid,
-                transfer_syntax,
-                dataset,
-                instances,
-                workers,
-                chunk_size
-            )
+            image_data_list = self._list_image_data(instances)
+            dataset = instances[0].dataset.as_tiled_full(image_data_list)
+            with WsiDicomFileWriter(filepath) as wsi_file:
+                wsi_file.write(
+                    uid,
+                    transfer_syntax,
+                    dataset,
+                    image_data_list,
+                    workers,
+                    chunk_size,
+                    offset_table
+                )
             filepaths.append(filepath)
         return filepaths
 
-    @classmethod
-    def _write_file(
-        cls,
-        filepath: Path,
-        uid: UID,
-        transfer_syntax: UID,
-        dataset: WsiDataset,
-        instances: List[WsiInstance],
-        workers: int,
-        chunk_size: int
-    ) -> None:
-        with WsiDicomFileWriter(filepath) as wsi_file:
-            wsi_file.write_preamble()
-            wsi_file.write_file_meta(uid, transfer_syntax)
-            dataset.SOPInstanceUID = uid
-            wsi_file.write_base(dataset)
-            frames = dataset.NumberOfFrames
-            bot_start, bot_end = wsi_file.write_pixel_data_start(frames)
-            frame_positions: List[int] = []
-            for (path, z), image_data in cls._list_image_data(instances):
-                frame_positions += wsi_file.write_pixel_data(
-                    image_data,
-                    z,
-                    path,
-                    workers,
-                    chunk_size
-                )
-            wsi_file.write_pixel_data_end()
-            wsi_file.write_bot(bot_start, bot_end, frame_positions)
-            wsi_file.close()
-
 
 class WsiDicomLevel(WsiDicomGroup):
     """Represents a level in the pyramid and contains one or more instances
@@ -653,7 +608,7 @@ class WsiDicomLevel(WsiDicomGroup):
     """
     def __init__(
         self,
-        instances: List[WsiInstance],
+        instances: Sequence[WsiInstance],
         base_pixel_spacing: SizeMm
     ):
         """Create a level from list of WsiInstances. Asign the pyramid level
@@ -661,7 +616,7 @@ class WsiDicomLevel(WsiDicomGroup):
 
         Parameters
         ----------
-        instances: List[WsiInstance]
+        instances: Sequence[WsiInstance]
             Instances to build the level.
         base_pixel_spacing: SizeMm
             Pixel spacing of base level.
@@ -717,13 +672,13 @@ class WsiDicomLevel(WsiDicomGroup):
     @classmethod
     def open(
         cls,
-        instances: List[WsiInstance],
+        instances: Sequence[WsiInstance],
     ) -> List['WsiDicomLevel']:
         """Return list of levels created wsi files.
 
         Parameters
         ----------
-        files: List[WsiInstance]
+        files: Sequence[WsiInstance]
             Instances to create levels from.
 
         Returns
@@ -734,25 +689,31 @@ class WsiDicomLevel(WsiDicomGroup):
         """
         levels: List['WsiDicomLevel'] = []
         instances_grouped_by_level = cls._group_instances(instances)
-
         base_group = list(instances_grouped_by_level.values())[0]
         base_pixel_spacing = base_group[0].pixel_spacing
         for level in instances_grouped_by_level.values():
             levels.append(cls(level, base_pixel_spacing))
-
         return levels
 
-    def matches(self, other_group: 'WsiDicomGroup') -> bool:
-        """Check if group is valid (Uids and tile size match).
-        The common Uids should match for all series. For level series the tile
-        size should also match. It is assumed that the instances in the groups
-        are matching each other.
+    def matches(self, other_level: 'WsiDicomGroup') -> bool:
+        """Check if level matches other level. If strict common Uids should
+        match. Wsi type and tile size should always match.
+
+        Parameters
+        ----------
+        other_level: WsiDicomGroup
+            Other level to match against.
+
+        Returns
+        ----------
+        bool
+            True if other level matches.
         """
-        other_instance = other_group.default_instance
-        this_instance = self.default_instance
+        other_level = cast(WsiDicomLevel, other_level)
         return (
-            other_group.uids == self.uids and
-            other_instance.tile_size == this_instance.tile_size
+            self.uids.matches(other_level.uids) and
+            other_level.wsi_type == self.wsi_type and
+            other_level.tile_size == self.tile_size
         )
 
     def get_highest_level(self) -> int:
@@ -802,7 +763,7 @@ class WsiDicomLevel(WsiDicomGroup):
                 f"Region {cropped_region}", f"level size {self.size}"
             )
         image = self.get_region(cropped_region, z, path)
-        tile_size = cropped_region.size/scale
+        tile_size = cropped_region.size.ceil_div(scale)
         image = image.resize(
             tile_size.to_tuple(),
             resample=Image.BILINEAR
@@ -879,17 +840,21 @@ class WsiDicomLevel(WsiDicomGroup):
     def create_child(
         self,
         scale: int,
+        output_path: Path,
         uid_generator: Callable[..., UID],
         workers: int,
-        chunk_size: int
+        chunk_size: int,
+        offset_table: Optional[str]
     ) -> 'WsiDicomLevel':
         """Creates a new WsiDicomLevel from this level by scaling the image
-        data. File is saved in same folder as parent.
+        data.
 
         Parameters
         ----------
         scale: int
             Scale factor.
+        output_path: Path
+            The path to write child to.
         uid_generator: Callable[..., UID]
             Uid generator to use.
         workers: int
@@ -897,6 +862,9 @@ class WsiDicomLevel(WsiDicomGroup):
         chunk_size: int
             Chunk size (number of tiles) to process at a time. Actual chunk
             size also depends on minimun_chunk_size from image_data.
+        offset_table: Optional[str]
+            Offset table to use, 'bot' basic offset table, 'eot' extended
+            offset table, None - no offset table.
 
         Returns
         ----------
@@ -915,36 +883,28 @@ class WsiDicomLevel(WsiDicomGroup):
             raise NotImplementedError(
                 "Can only construct pyramid from DICOM WSI files"
             )
-        output_path = (
-            self.default_instance.image_data._files[0].filepath.parent
-        )
-        new_image_size = self.default_instance.size / scale
+
         for instances in self._group_instances_to_file():
             uid = uid_generator()
             filepath = Path(os.path.join(output_path, uid + '.dcm'))
             transfer_syntax = instances[0].image_data.transfer_syntax
-            dataset = deepcopy(instances[0].dataset)
-            frames = self._get_number_of_frames(instances)
-            # Modify dataset to reflect scaled data
-            frames = frames // (scale*scale)
-            dataset.NumberOfFrames = frames
-            dataset.TotalPixelMatrixColumns = new_image_size.width
-            dataset.TotalPixelMatrixRows = new_image_size.height
-
-            new_pixel_spacing = self.pixel_spacing * scale
-            (
-                dataset.SharedFunctionalGroupsSequence[0].
-                PixelMeasuresSequence[0].PixelSpacing
-            ) = list(new_pixel_spacing.to_tuple())
-            self._write_file(
-                filepath,
-                uid,
-                transfer_syntax,
-                dataset,
-                instances,
-                workers,
-                chunk_size
+            image_data_list = self._list_image_data(instances)
+            dataset = instances[0].dataset.as_tiled_full(
+                image_data_list,
+                scale
             )
+
+            with WsiDicomFileWriter(filepath) as wsi_file:
+                wsi_file.write(
+                    uid,
+                    transfer_syntax,
+                    dataset,
+                    image_data_list,
+                    workers,
+                    chunk_size,
+                    offset_table,
+                    scale
+                )
             filepaths.append(filepath)
 
         created_instances = WsiInstance.open(
@@ -960,15 +920,15 @@ class WsiDicomSeries(metaclass=ABCMeta):
     pyramidal levels, lables, or overviews.
     """
 
-    def __init__(self, groups: List[WsiDicomGroup]):
+    def __init__(self, groups: Sequence[WsiDicomGroup]):
         """Create a WsiDicomSeries from list of WsiDicomGroups.
 
         Parameters
         ----------
-        groups: List[WsiDicomGroup]
+        groups: Sequence[WsiDicomGroup]
             List of groups to include in the series.
         """
-        self._groups: List[WsiDicomGroup] = groups
+        self._groups = groups
 
         if len(self.groups) != 0 and self.groups[0].uids is not None:
             self._uids = self._validate_series(self.groups)
@@ -1004,15 +964,15 @@ class WsiDicomSeries(metaclass=ABCMeta):
     def wsi_type(self) -> str:
         """Should return the wsi type of the series ('VOLUME', 'LABEL', or
         'OVERVIEW'"""
-        raise NotImplementedError
+        raise NotImplementedError()
 
     @property
-    def groups(self) -> List[WsiDicomGroup]:
+    def groups(self) -> Sequence[WsiDicomGroup]:
         """Return contained groups."""
         return self._groups
 
     @property
-    def uids(self) -> Optional[BaseUids]:
+    def uids(self) -> Optional[SlideUids]:
         """Return uids."""
         return self._uids
 
@@ -1050,13 +1010,13 @@ class WsiDicomSeries(metaclass=ABCMeta):
 
     @classmethod
     @abstractmethod
-    def open(cls, instances: List[WsiInstance]) -> 'WsiDicomSeries':
+    def open(cls, instances: Sequence[WsiInstance]) -> 'WsiDicomSeries':
         raise NotImplementedError
 
     def _validate_series(
             self,
-            groups: Union[List[WsiDicomGroup], List[WsiDicomLevel]]
-    ) -> Optional[BaseUids]:
+            groups: Union[Sequence[WsiDicomGroup], Sequence[WsiDicomLevel]]
+    ) -> Optional[SlideUids]:
         """Check that no files or instances in series is duplicate and that
         all groups in series matches.
         Raises WsiDicomMatchError otherwise.
@@ -1064,12 +1024,12 @@ class WsiDicomSeries(metaclass=ABCMeta):
 
         Parameters
         ----------
-        groups: Union[List[WsiDicomGroup], List[WsiDicomLevel]]
+        groups: Union[Sequence[WsiDicomGroup], Sequence[WsiDicomLevel]]
             List of groups or levels to check
 
         Returns
         ----------
-        Optional[BaseUids]:
+        Optional[SlideUids]:
             Matching uids
         """
         WsiDataset.check_duplicate_dataset(self.datasets, self)
@@ -1100,7 +1060,8 @@ class WsiDicomSeries(metaclass=ABCMeta):
         output_path: str,
         uid_generator: Callable[..., UID],
         workers: int,
-        chunk_size: int
+        chunk_size: int,
+        offset_table: Optional[str]
     ) -> List[Path]:
         """Save WsiDicomSeries as DICOM-files in path.
 
@@ -1114,6 +1075,9 @@ class WsiDicomSeries(metaclass=ABCMeta):
         chunk_size:
             Chunk size (number of tiles) to process at a time. Actual chunk
             size also depends on minimun_chunk_size from image_data.
+        offset_table: Optional[str] = 'bot'
+            Offset table to use, 'bot' basic offset table, 'eot' extended
+            offset table, None - no offset table.
 
         Returns
         ----------
@@ -1126,7 +1090,8 @@ class WsiDicomSeries(metaclass=ABCMeta):
                 output_path,
                 uid_generator,
                 workers,
-                chunk_size
+                chunk_size,
+                offset_table
             )
             filepaths.extend(group_file_paths)
         return filepaths
@@ -1134,17 +1099,22 @@ class WsiDicomSeries(metaclass=ABCMeta):
 
 class WsiDicomLabels(WsiDicomSeries):
     """Represents a series of WsiDicomGroups of the label wsi flavor."""
+    WSI_TYPE = 'LABEL'
+
     @property
     def wsi_type(self) -> str:
-        return 'LABEL'
+        return self.WSI_TYPE
 
     @classmethod
-    def open(cls, instances: List[WsiInstance]) -> 'WsiDicomLabels':
+    def open(
+        cls,
+        instances: Sequence[WsiInstance]
+    ) -> 'WsiDicomLabels':
         """Return labels created from wsi files.
 
         Parameters
         ----------
-        instances: List[WsiInstance]
+        instances: Sequence[WsiInstance]
             Instances to create labels from.
 
         Returns
@@ -1158,17 +1128,22 @@ class WsiDicomLabels(WsiDicomSeries):
 
 class WsiDicomOverviews(WsiDicomSeries):
     """Represents a series of WsiDicomGroups of the overview wsi flavor."""
+    WSI_TYPE = 'OVERVIEW'
+
     @property
     def wsi_type(self) -> str:
-        return 'OVERVIEW'
+        return self.WSI_TYPE
 
     @classmethod
-    def open(cls, instances: List[WsiInstance]) -> 'WsiDicomOverviews':
+    def open(
+        cls,
+        instances: Sequence[WsiInstance]
+    ) -> 'WsiDicomOverviews':
         """Return overviews created from wsi files.
 
         Parameters
         ----------
-        instances: List[WsiInstance]
+        instances: Sequence[WsiInstance]
             Instances to create overviews from.
 
         Returns
@@ -1183,17 +1158,22 @@ class WsiDicomOverviews(WsiDicomSeries):
 class WsiDicomLevels(WsiDicomSeries):
     """Represents a series of WsiDicomGroups of the volume (e.g. pyramidal
     level) wsi flavor."""
+    WSI_TYPE = 'VOLUME'
+
     @property
     def wsi_type(self) -> str:
-        return 'VOLUME'
+        return self.WSI_TYPE
 
     @classmethod
-    def open(cls, instances: List[WsiInstance]) -> 'WsiDicomLevels':
+    def open(
+        cls,
+        instances: Sequence[WsiInstance]
+    ) -> 'WsiDicomLevels':
         """Return overviews created from wsi files.
 
         Parameters
         ----------
-        instances: List[WsiInstance]
+        instances: Sequence[WsiInstance]
             Instances to create levels from.
 
         Returns
@@ -1204,12 +1184,12 @@ class WsiDicomLevels(WsiDicomSeries):
         levels = WsiDicomLevel.open(instances)
         return cls(levels)
 
-    def __init__(self, levels: List[WsiDicomLevel]):
+    def __init__(self, levels: Sequence[WsiDicomLevel]):
         """Holds a stack of levels.
 
         Parameters
         ----------
-        levels: List[WsiDicomLevel]
+        levels: Sequence[WsiDicomLevel]
             List of levels to include in series
         """
         self._levels = OrderedDict(
@@ -1380,7 +1360,9 @@ class WsiDicomLevels(WsiDicomSeries):
         highest_level: int,
         uid_generator: Callable[..., UID] = generate_uid,
         workers: Optional[int] = None,
-        chunk_size: int = 100
+        chunk_size: int = 100,
+        offset_table: Optional[str] = 'bot',
+        add_to_excisting: bool = True
     ) -> List[Path]:
         """Construct missing pyramid levels from excisting levels.
 
@@ -1394,6 +1376,11 @@ class WsiDicomLevels(WsiDicomSeries):
         chunk_size: int = 100
             Chunk size (number of tiles) to process at a time. Actual chunk
             size also depends on minimun_chunk_size from image_data.
+        offset_table: Optional[str] = 'bot'
+            Offset table to use, 'bot' basic offset table, 'eot' extended
+            offset table, None - no offset table.
+        add_to_excisting: bool = True
+            If to add the created levels to excisting levels.
 
         Returns
         ----------
@@ -1414,14 +1401,20 @@ class WsiDicomLevels(WsiDicomSeries):
                 # Find the closest larger level for missing level
                 closest_level = self.get_closest_by_level(pyramid_level)
                 # Create scaled level
+                output_path = closest_level.files[0].parent
                 new_level = closest_level.create_child(
                     scale=2,
+                    output_path=output_path,
                     uid_generator=uid_generator,
                     workers=workers,
-                    chunk_size=chunk_size
+                    chunk_size=chunk_size,
+                    offset_table=offset_table
                 )
                 # Add level to available levels
-                self._levels[new_level.level] = new_level
+                if add_to_excisting:
+                    self._levels[new_level.level] = new_level
+                else:
+                    new_level.close()
                 filepaths += new_level.files
         return filepaths
 
@@ -1434,8 +1427,7 @@ class WsiDicom:
         levels: WsiDicomLevels,
         labels: WsiDicomLabels,
         overviews: WsiDicomOverviews,
-        annotations: List[AnnotationInstance] = []
-
+        annotations: Sequence[AnnotationInstance] = []
     ):
         """Holds wsi dicom levels, labels and overviews.
 
@@ -1447,17 +1439,16 @@ class WsiDicom:
             Series of label images.
         overviews: WsiDicomOverviews
             Series of overview images
-        annotations: List[AnnotationInstance] = []
+        annotations: Sequence[AnnotationInstance] = []
             Sup-222 annotation instances.
         """
-
         self._levels = levels
         self._labels = labels
         self._overviews = overviews
         self.annotations = annotations
 
         self.uids = self._validate_collection(
-            [self.levels, self.labels, self.overviews],
+            [self.levels, self.labels, self.overviews]
         )
 
         self.optical = OpticalManager.open(
@@ -1466,7 +1457,7 @@ class WsiDicom:
 
         if self.annotations != []:
             for annotation in self.annotations:
-                if annotation.base_uids != self.uids:
+                if annotation.slide_uids != self.uids:
                     warnings.warn("Annotations uids does not match")
 
         self.__enter__()
@@ -1566,14 +1557,14 @@ class WsiDicom:
     @classmethod
     def open(
         cls,
-        path: Union[str, List[str], Path, List[Path]]
+        path: Union[str, Sequence[str], Path, Sequence[Path]]
     ) -> 'WsiDicom':
         """Open valid wsi dicom files in path and return a WsiDicom object.
         Non-valid files are ignored.
 
         Parameters
         ----------
-        path: Union[str, List[str], Path, List[Path]]
+        path: Union[str, Sequence[str], Path, Sequence[Path]]
             Path to files to open
 
         Returns
@@ -1591,27 +1582,26 @@ class WsiDicom:
             sop_class_uid = cls._get_sop_class_uid(filepath)
             if sop_class_uid == WSI_SOP_CLASS_UID:
                 wsi_file = WsiDicomFile(filepath)
-                if(wsi_file.wsi_type == 'VOLUME'):
+                if(wsi_file.wsi_type == WsiDicomLevels.WSI_TYPE):
                     level_files.append(wsi_file)
-                elif(wsi_file.wsi_type == 'LABEL'):
+                elif(wsi_file.wsi_type == WsiDicomLabels.WSI_TYPE):
                     label_files.append(wsi_file)
-                elif(wsi_file.wsi_type == 'OVERVIEW'):
+                elif(wsi_file.wsi_type == WsiDicomOverviews.WSI_TYPE):
                     overview_files.append(wsi_file)
                 else:
                     wsi_file.close()
             elif sop_class_uid == ANN_SOP_CLASS_UID:
                 annotation_files.append(filepath)
-
         base_dataset = cls._get_base_dataset(level_files)
-        base_uids = base_dataset.base_uids
+        slide_uids = base_dataset.uids.slide
         base_tile_size = base_dataset.tile_size
         level_instances = WsiInstance.open(
             level_files,
-            base_uids,
+            slide_uids,
             base_tile_size
         )
-        label_instances = WsiInstance.open(label_files, base_uids)
-        overview_instances = WsiInstance.open(overview_files, base_uids)
+        label_instances = WsiInstance.open(label_files, slide_uids)
+        overview_instances = WsiInstance.open(overview_files, slide_uids)
 
         levels = WsiDicomLevels.open(level_instances)
         labels = WsiDicomLabels.open(label_instances)
@@ -1928,7 +1918,8 @@ class WsiDicom:
         output_path: str,
         uid_generator: Callable[..., UID] = generate_uid,
         workers: Optional[int] = None,
-        chunk_size: Optional[int] = None
+        chunk_size: Optional[int] = None,
+        offset_table: Optional[str] = 'bot'
     ) -> List[Path]:
         """Save wsi as DICOM-files in path. Instances for the same pyramid
         level will be combined when possible to one file (e.g. not split
@@ -1946,6 +1937,9 @@ class WsiDicom:
         chunk_size: Optional[int] = None
             Chunk size (number of tiles) to process at a time. Actual chunk
             size also depends on minimun_chunk_size from image_data.
+        offset_table: Optional[str] = 'bot'
+            Offset table to use, 'bot' basic offset table, 'eot' extended
+            offset table, None - no offset table.
 
         Returns
         ----------
@@ -1971,7 +1965,8 @@ class WsiDicom:
                 output_path,
                 uid_generator,
                 workers,
-                chunk_size
+                chunk_size,
+                offset_table
             )
             filepaths.extend(collection_filepaths)
         return filepaths
@@ -1982,7 +1977,7 @@ class WsiDicom:
         return metadata.MediaStorageSOPClassUID
 
     @staticmethod
-    def _get_filepaths(path: Union[str, List[str], Path, List[Path]]):
+    def _get_filepaths(path: Union[str, Sequence[str], Path, Sequence[Path]]):
         """Return file paths to files in path.
         If path is folder, return list of folder files in path.
         If path is single file, return list of that path.
@@ -1991,7 +1986,7 @@ class WsiDicom:
 
         Parameters
         ----------
-        path: path: Union[str, List[str], Path, List[Path]]
+        path: path: Union[str, Sequence[str], Path, Sequence[Path]]
             Path to folder, file or list of files
 
         Returns
@@ -2017,13 +2012,13 @@ class WsiDicom:
 
     @staticmethod
     def _get_base_dataset(
-        files: List[WsiDicomFile]
+        files: Sequence[WsiDicomFile]
     ) -> WsiDataset:
         """Return file with largest image (width) from list of files.
 
         Parameters
         ----------
-        files: List[WsiDicomFile]
+        files: Sequence[WsiDicomFile]
            List of files.
 
         Returns
@@ -2040,12 +2035,12 @@ class WsiDicom:
         return base_dataset
 
     @staticmethod
-    def _filter_paths(filepaths: List[Path]) -> List[Path]:
+    def _filter_paths(filepaths: Sequence[Path]) -> List[Path]:
         """Filter list of paths to only include valid dicom files.
 
         Parameters
         ----------
-        filepaths: List[Path]
+        filepaths: Sequence[Path]
             Paths to filter
 
         Returns
@@ -2059,32 +2054,32 @@ class WsiDicom:
 
     def _validate_collection(
         self,
-        series: List[WsiDicomSeries]
-    ) -> BaseUids:
-        """Check that no files or instance in collection is duplicate, that all
-        series have the same base uids. Raises WsiDicomMatchError otherwise.
-        Returns base uid for collection.
+        series: Sequence[WsiDicomSeries]
+    ) -> SlideUids:
+        """Check that no files or instance in collection is duplicate, and, if
+        strict, that all series have the same base uids.
+        Raises WsiDicomMatchError otherwise. Returns base uid for collection.
 
         Parameters
         ----------
-        series: List[WsiDicomSeries]
-            List of series to check
+        series: Sequence[WsiDicomSeries]
+            List of series to check.
 
         Returns
         ----------
-        BaseUids
+        SlideUids
             Matching uids
         """
         WsiDataset.check_duplicate_dataset(self.datasets, self)
         WsiInstance.check_duplicate_instance(self.instances, self)
 
         try:
-            base_uids = next(
+            slide_uids = next(
                 item.uids for item in series if item.uids is not None
             )
         except StopIteration:
             raise WsiDicomNotFoundError("Valid series", "in collection")
         for item in series:
-            if item.uids is not None and item.uids != base_uids:
+            if item.uids is not None and item.uids != slide_uids:
                 raise WsiDicomMatchError(str(item), str(self))
-        return base_uids
+        return slide_uids
